/*		
 *	Exemple d'utilisation de console sous windows	
 *	
 *	Auteur	:	Alexandre Abreu, 
 *	Date	:	11/2001
 *	
 *	Le contenu de ce fichier est exempt de tout droit, vous pouvez le ripper,
 *	le modifier, l'utiliser a vos souhaits.	
 */



/////////////////////////////////////////////////////////////////////////////////////////
///  	headers

#include <windows.h>
#include <stdio.h>		// pour les print()
#include <assert.h>		// pour assert()
#include <conio.h>		// pour _kbhit()


/////////////////////////////////////////////////////////////////////////////////////////
///  	nouveau types et macros


// petit mecanisme d'erreur
enum eErrorState {
	
	ERR_OK,
	ERR_ALLOC_CONSOLE	=	-1,
	ERR_GET_STD			=	ERR_ALLOC_CONSOLE - 1,
	ERR_ALREADY_INIT	=	ERR_GET_STD - 1,
	ERR_GET_STD_HANDLE	=	ERR_ALREADY_INIT - 1,
	ERR_NO_CONSOLE		=	ERR_GET_STD_HANDLE - 1

};


// indique le type de texte a ecrire
enum ePrintType {
	
	PRINT_TYPE_ERROR,
	PRINT_TYPE_WARNING,
	PRINT_TYPE_COMMENT
};


// structure console regroupant les informations pertinentes sur une console
typedef struct sConsole {
	
	int			m_bInit;
	HANDLE		m_hStdOut, m_hStdIn, m_hStdErr;
} sConsole;


// une macro d'exemple nous permettant d'ecrire dans la console
#define PRINT(type,console,msg)		Print( type, console, __LINE__, __FILE__, msg )




/////////////////////////////////////////////////////////////////////////////////////////
///  	variables globales

// une variable globale de type console
sConsole g_Console;




/////////////////////////////////////////////////////////////////////////////////////////
///  	fonctions 




//	Fonction : InitConsole
//	Alloue une console pour une application win32, 
//	rempli la structure passee en parametre avec les handles 
//	des sorties standards, etc ... associees la console

eErrorState InitConsole( sConsole * lpConsole )
{	
	assert(lpConsole);
	
	// on rappel qu'il n'y a qu'une console par application win32, 
	// dons si il y a deja eu initialisation on sort.
	if( lpConsole->m_bInit ){
		
		return ERR_ALREADY_INIT;
	}
	
	// on alloue une console pour notre application
	if( AllocConsole() != TRUE ){
		
		return ERR_ALLOC_CONSOLE;
	}
	
	///////////////
	//	On recupere les handles associes a cette console
	//	

	//	STDIN

	lpConsole->m_hStdIn		= GetStdHandle(STD_INPUT_HANDLE);
	
	if( lpConsole->m_hStdIn == INVALID_HANDLE_VALUE ){
		
		return ERR_GET_STD_HANDLE;
	}
	
	// STDOUT

	lpConsole->m_hStdOut	= GetStdHandle(STD_OUTPUT_HANDLE);
		
	if( lpConsole->m_hStdOut == INVALID_HANDLE_VALUE ){
		
		return ERR_GET_STD_HANDLE;
	}
	
	// STDERR

	lpConsole->m_hStdErr	= GetStdHandle(STD_ERROR_HANDLE);	
	
	if( lpConsole->m_hStdErr == INVALID_HANDLE_VALUE ){
		
		return ERR_GET_STD_HANDLE;
	}
	
	// initialisation ok
	lpConsole->m_bInit = 1;

	return ERR_OK;		
}




//	Fonction : CloseConsole
//	Libere une console deja allouee

eErrorState CloseConsole( sConsole * lpConsole )
{
	assert(lpConsole);
	
	// on ne libere une console que si elle a deja ete initialisee
	if( !lpConsole->m_bInit ){
		
		return ERR_NO_CONSOLE;
	}
	
	FreeConsole();
	
	// on utilise la macro ZeroMemory pour reinitialiser notre structure
	// (equivalent a un memset( lpConsole, 0, sizeof(sConsole) )
	ZeroMemory( lpConsole, sizeof(sConsole));
	
	return ERR_OK;
}




//	Fonction : Print
//	Un exemple de fonction Print tres simple adaptee a l'ecriture dans une console.

int Print( ePrintType type, sConsole * lpConsole, int iLine, char *szFile, char *szMsg, ... )
{
	char szMessage[256], szMessage_tmp[256];
	va_list paramList;
	int ret = 0;
	
	assert( szFile && szMsg && lpConsole );
	
	sprintf( szMessage, "File : %s, line : %d  -  ", szFile, iLine );
	
	va_start(paramList, szMsg);
	vsprintf(szMessage_tmp, szMsg, paramList);
	va_end( paramList );    
	
	strncat( szMessage, szMessage_tmp, sizeof(szMessage) );
	
	if ( lpConsole->m_bInit ){

		switch( type ){

		case PRINT_TYPE_WARNING:
			
			SetConsoleTextAttribute( g_Console.m_hStdOut, FOREGROUND_INTENSITY | BACKGROUND_BLUE);			
			break;
			
		case PRINT_TYPE_COMMENT:

			SetConsoleTextAttribute( g_Console.m_hStdOut, FOREGROUND_GREEN);
			break;

		case PRINT_TYPE_ERROR:

			SetConsoleTextAttribute( g_Console.m_hStdOut, FOREGROUND_INTENSITY | BACKGROUND_RED);
			break;
		}

		ret = printf(szMessage);

	}

	return ret;		
}







//	Fonction : WinMain
//	Point d'entree de notre programme

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	////////////////
	// INIT CONSOLE
	
	// on initialise notre console
	InitConsole( &g_Console );
	
	// on lui change le titre 
	SetConsoleTitle("Console d'exemple : Alexandre Abreu, Programmation windows chap.6");
	
	// on redirige la sortie standard vers la sortie console
	FILE *out = freopen("CONOUT$", "wb", stdout);
	
	
	////////////////
	// ECRITURE CONSOLE
	
	// on teste que la re-direction fonctionne avec deux fonctions standards printf et fprintf
	fprintf(out, "On ecrit dans la console avec un fprintf\n");
	printf("On ecrit dans la console avec un print\n");
	
	// on teste notre macro PRINT pour ecrire dans la console
	PRINT( PRINT_TYPE_COMMENT, &g_Console, "Notre macro PRINT : comment\n");
	PRINT( PRINT_TYPE_WARNING, &g_Console, "Notre macro PRINT : test WARNING\n");
	PRINT( PRINT_TYPE_ERROR, &g_Console,   "Notre macro PRINT : test ERROR\n");
	
	// on attend qu'une touche soit tapee au clavier
	while( !_kbhit() ){}
	
	
	///////////////
	// FIN

	// on ferme la console
	CloseConsole( &g_Console );
	
	return 0;
}
